import mutagen
import os
import time
import re
import shutil

from mutagen.mp4 import MP4
from mutagen.mp4 import MP4Tags
from mutagen.mp4 import MP4Info
from mutagen.easymp4 import EasyMP4
from mutagen.easymp4 import EasyMP4Tags

# Author: Anthony Stahler
# Last updated: Jan 13, 2024
# License: Free for personal use, for commercial use please reach out to the author via the BestEdit contact page: http://bestedit.net/contact/
# The most recent version of this script can be found at http://bestedit.net/resources/
#
# To use this script you need python and the mutagen library, which can be installed on the command line like this:
# python3 -m pip install mutagen
#
# This script will take a few variables and fill in the rest of the data based on the file name so you get pretty names on your files
# Example file name formats:
#
# My.Lovely.Liar.S01E14-16.1080p.bestedit.ver2.mp4
# Makes this description: My Lovely Liar S01E14-16 by BestEdit, revision 2
#
# Please note this doesn't get rid of the auto-generated title, seasonNumber, episodeNumber, showName metadata from VLC, that's created in realtime by this file:
# C:\Program Files\VideoLAN\VLC\lua\meta\reader\filename.luac
# and you can move or delete that file to make VLC stop auto-interpreting your file names
#
# CAUTION: This doesn't remove certain metadata like the Encoded Date and Tagged Date, although ffmpeg usually doesn't write/transfer those fields unless you tell it to. But if you're exporting directly from something like Adobe Premiere they may be present.
# Some custom metadata like Encoded Date can also be wiped out in Windows file properties by using Remove Properties and Personal Information - however some fields like Tagged Date will persist through this operation.
# Examples:
# exif 36868 = date time digitized
# basic, creation date (media created)
# You can use MediaInfo to check for those or other metadata present in your encoded files.
#
# Be aware that these metadata updates can be slow, for some reason it seems to rewrite the entire file for certain formats, and does a quick update for others, so it depends on your library.
# IMPORTANT: if you are not using the copydir, do not stop this while it's running - you'll leave whatever file mutagen is currently writing in an inconsistent/truncated state (which you can see in MediaInfo)
# So if you're unsure about the speed try it on a small number of files first, or use the copy feature and delete your old files afterward. Copying may be slower because it does a full copy of everything, but you can be sure all data is intact before erasing old files.
#


# Variables
# --------------------
groupname = "BestEdit"
encodedby = "" # Fill this in if you're encoding someone else's file, otherwise leave blank, is appended to Artist metadata

# path to your files - any \ in path should be written as \\
removemetadir = "S:\\metadata-tests\\"
#copydir = "" # Only fill in if you want to make a copy rather than overwriting meta on original files, WARNING: will overwrite files without asking!
copydir = removemetadir + "done\\"

# --------------------
targetext = ".mp4"
resolutions = ["720p", "1080p", "4k"] # these are the resolution strings in the filename.1080p.mp4 movie name used to split it

directory = os.fsencode(removemetadir)
yearcheck = re.compile(".*\s[0-9]{4}$")
revcheck = re.compile(".*\.ver([0-9]+)\..*", flags=re.I)

#print('mp4 tags:', *sorted(EasyMP4Tags.Set.keys()), end='\n\n')

for file in os.listdir(directory):
	filename = os.fsdecode(file)
	if filename.endswith(targetext): 
		splitfound = ""

		# first figure out what text we want to use with a few regular expressions on the title
		for resstr in resolutions:
			if filename.lower().find("." + resstr.lower() + ".") > -1 :
				splitfound = "\\." + resstr.lower() + "\\."

		if splitfound != "":
			splitname = re.split(splitfound, filename, maxsplit=0, flags=re.I) # using regex for case insensitive
			splitname[0] = splitname[0].replace("."," ")
			
			#print(splitname)

			#filename is in title.year.resolution format, pretty format the year
			if yearcheck.match(splitname[0]):
				tmpyear = splitname[0][-4:]
				splitname[0] = splitname[0][0:-4] + "(" + tmpyear + ")"

			newtitle = splitname[0]
			newdesc = newtitle + " by " + groupname
			newartist = groupname

			if encodedby != "":
				newartist = newartist + " encoded by " + encodedby # changing this to use the proper encoded by meta field

			# filename has version number in title
			if revcheck.match(splitname[1]):
				newdesc = newdesc + ", revision " + revcheck.match(splitname[1]).group(1)
		
			print("New description will be: " + newdesc)
			##print(newtitle)
			##print(newartist)
		else:
			print("No split found!")
		

		# Now open the MP4 to wipe the messy encoder metadata and apply our new pretty formatting
		tmpmp4 = MP4(removemetadir + filename)

		print('Adusting metadata:', filename, end='\n\n')

		blanktags = MP4Tags()
		blanktags['\xa9nam'] = newtitle
		blanktags['\xa9ART'] = newartist 
		
		blanktags['desc'] = newdesc
		#blanktags["----:TXXX:Tagged date"] = bytes("silly", 'UTF-8') # Doesn't work, appends new metadata with same name, doesn't overwrite old one
		#blanktags["----:TXXX:"] = bytes("silly", 'UTF-8') # Doesn't work
		
		if encodedby != "":
			blanktags['\xa9too'] = encodedby # Shows up in the Encoded By field in VLC

			#blanktags["----:EncodedBy"] = bytes(encodedby, 'UTF-8') # doesn't like this format, errors out
			#blanktags["----:Microsoft:EncodedBy"] = bytes(encodedby, 'UTF-8') # this works but doesn't show up in Microsft properties
			#blanktags["----:Quicktime:EncodedBy"] = bytes(encodedby, 'UTF-8') # this works but doesn't show up in Microsft properties
			#blanktags["----:ItemList:EncodedBy"] = bytes(encodedby, 'UTF-8') # shows up as Encoded By in Mediainfo (as does any other prefix instead of ItemList) but doesn't show up in Microsft properties
			#blanktags["----:com.apple.iTunes:EncodedBy"] = bytes(encodedby, 'UTF-8') # this works but doesn't show up in Microsft properties

			#blanktags["----:com.apple.iTunes:DateTimeDigitized"] = bytes("sillyMe2", 'UTF-8') 
			#blanktags["----:ItemList:Tagged date"] = bytes("sillyMe", 'UTF-8') # this works but doesn't show up in Microsft properties

		tmpmp4.tags = blanktags

		if copydir != "":
			# since mutagen expects a full mp4 to apply changes to we need to manually copy before saving out these tags to the new target file
			shutil.copyfile(removemetadir + filename, copydir + filename)
			tmpmp4.save(copydir + filename)
		else:
			tmpmp4.tags.save(removemetadir + filename)

		#print('info:', tmpmp4.info.pprint(), end='\n\n')
		#print('tags:', tmpmp4.tags.pprint(), end='\n\n')
		#print('file:', tmpmp4.pprint(), end='\n\n')

		continue
	else:
		continue
